<?php
/**
 * SplClassLoader implementation that implements the technical interoperability
 * standards for PHP 5.3 namespaces and class names.
 *
 * Modified to pass WP valid theme check and usable by PHP 5.2
 *
 * http://groups.google.com/group/php-standards/web/final-proposal
 *
 *     // Example which loads classes for the Doctrine Common package in the
 *     // Doctrine\Common namespace.
 *     $classLoader = new SplClassLoader('Doctrine\Common', '/path/to/doctrine');
 *     $classLoader->register();
 *
 * @author jason.xie@victheme.com
 * @author Jonathan H. Wage <jonwage@gmail.com>
 * @author Roman S. Borschel <roman@code-factory.org>
 * @author Matthew Weier O'Phinney <matthew@zend.com>
 * @author Kris Wallsmith <kris.wallsmith@gmail.com>
 * @author Fabien Potencier <fabien.potencier@symfony-project.org>
 */
class VTCore_Autoloader {

  private $_fileExtension = '.php';
  private $_namespace;
  private $_includePath;
  private $_namespaceSeparator = '_';
  private $_remove = '';
  static protected $cache = array();
  static protected $paths = array();

  /**
   * Creates a new <tt>SplClassLoader</tt> that loads classes of the
   * specified namespace.
   *
   * @param string $ns The namespace to use.
   */
  public function __construct($ns = null, $includePath = null) {
    $this->_namespace = $ns;
    $this->_includePath = $includePath;
  }

  /**
   * Sets the namespace separator used by classes in the namespace of this class loader.
   *
   * @param string $sep The separator to use.
   */
  public function setNamespaceSeparator($sep) {
    $this->_namespaceSeparator = $sep;
  }

  /**
   * Gets the namespace seperator used by classes in the namespace of this class loader.
   *
   * @return void
   */
  public function getNamespaceSeparator() {
    return $this->_namespaceSeparator;
  }

  /**
   * Sets the base include path for all class files in the namespace of this class loader.
   *
   * @param string $includePath
   */
  public function setIncludePath($includePath) {
    $this->_includePath = $includePath;
  }

  /**
   * Sets the string to remove from path;
   *
   * @param string $includePath
   */
  public function setRemovePath($remove) {
    $this->_remove = $remove;
  }

  /**
   * Gets the base include path for all class files in the namespace of this class loader.
   *
   * @return string $includePath
   */
  public function getIncludePath() {
    return $this->_includePath;
  }

  /**
   * Sets the file extension of class files in the namespace of this class loader.
   *
   * @param string $fileExtension
   */
  public function setFileExtension($fileExtension) {
    $this->_fileExtension = $fileExtension;
  }

  /**
   * Gets the file extension of class files in the namespace of this class loader.
   *
   * @return string $fileExtension
   */
  public function getFileExtension() {
    return $this->_fileExtension;
  }

  /**
   * Installs this class loader on the SPL autoload stack.
   */
  public function register() {
    spl_autoload_register(array($this, 'loadClass'));
  }

  /**
   * Uninstalls this class loader from the SPL autoloader stack.
   */
  public function unregister() {
    spl_autoload_unregister(array($this, 'loadClass'));
  }


  public static function setMapCache($map) {
    self::$cache = $map;
  }


  public static function resetMapCache() {
    self::$cache = array();
  }

  public static function getMapCache() {
    return self::$cache;
  }


  /**
   * Loads the given class or interface.
   *
   * @param string $className The name of the class to load.
   * @return void
   */
  public function loadClass($className) {
    if (strpos($className, $this->_namespace) !== false) {
      if (isset(self::$cache[$className]) && !empty(self::$cache[$className])) {
        (include self::$cache[$className]) or $this->detectFile($className);
      }
      else {
        $this->detectFile($className);
      }
    }
  }

  /**
   * Detect the class file location
   * @param $className
   */
  public function detectFile($className) {

    if (NULL === $this->_namespace || $this->_namespace . $this->_namespaceSeparator === substr($className, 0, strlen($this->_namespace . $this->_namespaceSeparator))) {

      $key = $className;

      // Temporary convert double underscore.
      $className = str_replace('__', '##', $className);

      $fileName = '';
      $namespace = '';
      if (FALSE !== ($lastNsPos = strripos($className, $this->_namespaceSeparator))) {
        $namespace = substr($className, 0, $lastNsPos);
        $className = substr($className, $lastNsPos + 1);
        $fileName = str_replace($this->_namespaceSeparator, DIRECTORY_SEPARATOR, $namespace) . DIRECTORY_SEPARATOR;
      }

      $fileName .= str_replace('_', DIRECTORY_SEPARATOR, $className) . $this->_fileExtension;
      $file = ($this->_includePath !== NULL ? $this->_includePath . DIRECTORY_SEPARATOR : '') . strtolower($fileName);
      $file = str_replace(array($this->_remove, '##'), array('', '_',), $file);

      // Record whether the file exists or not
      $exists = is_file($file);

      if ($exists == true) {
        self::$cache[$key] = $file;
        include self::$cache[$key];
      }
    }
  }
}