<?php
/*
Plugin Name: VicTheme Timeline
Plugin URI: http://plugin.victheme.com/time-line/
Description: Plugin for creating time line element.
Author: jason.xie@victheme.com
Version: 2.0.1
Author URI: http://visualline.victheme.com/
Requires PHP: 7.0
*/


define('VTCORE_TIMELINE_CORE_VERSION', '1.8.0');
define('VTCORE_TIMELINE_VERSION', '2.0.0');


/**
 * Main class for initialiling the plugin
 *
 * Class VTCore_Timeline
 */
class VTCore_Timeline
{

    /**
     * Booting the plugin
     */
    public function __construct()
    {
        load_plugin_textdomain('victheme_timeline', false, 'victheme_timeline/languages');
        add_action('plugins_loaded', function () {
            if ($this->checkDependencies()) {
                $this->doAutoloader();
                $this->doAssets();
                $this->doActions();
                $this->doFilters();
                $this->doShortcodes();
                $this->doVisualComposer();
            }
        }, 11);
    }


    /**
     * Check if dependencies are met
     * @return bool
     */
    protected function checkDependencies()
    {
        // Check for VTCore availability
        if (!defined('VTCORE_VERSION')) {
            add_action('admin_notices', function() {
                $this->addNotice(__('VicTheme Timeline depends on VicTheme Core Plugin which is not activated or missing, Please enable it first before VicTheme Timeline can work properly.', 'victheme_timeline'));
            });
            return false;
        }


        // Check for VTCore version
        if (version_compare(VTCORE_VERSION, VTCORE_TIMELINE_CORE_VERSION, '!=')) {
            add_action('admin_notices', function() {
                $this->addNotice(sprintf(__('VicTheme Timeline depends on VicTheme Core Plugin API version %s to operate properly.', 'victheme_timeline'), VTCORE_TIMELINE_CORE_VERSION));
            });
            return false;
        }

        // Check for VisualComposer
        if (!defined('WPB_VC_VERSION')) {
            add_action('admin_notices', function () {
                $this->addNotice(__('Timeline requires Visual Composer Plugin enabled before it can function properly.', 'victheme_timeline'));
            });
            return false;
        }

        // Check for VisualComposer version
        if (!version_compare(WPB_VC_VERSION, '5.5.0', '>=')) {
            add_action('admin_notices', function () {
                $this->addNotice(__('TimeLine requires Visual Composer Plugin version 5.5.0 and above before it can function properly.', 'victheme_timeline'));
            });
            return false;
        }

        return true;
    }


    /**
     * Helper method to print notices
     * @param $text
     */
    protected function addNotice($text)
    {
        echo '<div class="notice error""><p>' . esc_html($text) . '</p></div>';
    }


    /**
     * Registering assets
     */
    protected function doAssets()
    {
        VTCore_Wordpress_Init::getFactory('assets')
            ->get('library')
            ->detect(dirname(__FILE__) . DIRECTORY_SEPARATOR . 'assets', plugins_url('', __FILE__) . '/assets', false, 'VTCore_Timeline_Assets');
    }


    /**
     * Registering autoloaders
     */
    protected function doAutoloader()
    {
        $autoloader = new VTCore_Autoloader('VTCore_Timeline', dirname(__FILE__));
        $autoloader->setRemovePath('vtcore' . DIRECTORY_SEPARATOR . 'timeline' . DIRECTORY_SEPARATOR);
        $autoloader->register();
    }


    /**
     * Registering shortcodes
     */
    protected function doShortcodes()
    {
        VTCore_Wordpress_Init::getFactory('shortcodes')
            ->merge([
                'timeend' => 'VTCore_Timeline_Shortcodes_TimeEnd',
                'timeevents' => 'VTCore_Timeline_Shortcodes_TimeEvents',
                'timeline' => 'VTCore_Timeline_Shortcodes_TimeLine',
                'timelinequery' => 'VTCore_Timeline_Shortcodes_TimeLineQuery',
                'timelinesimple' => 'VTCore_Timeline_Shortcodes_TimeLineSimple',
                'timemajor' => 'VTCore_Timeline_Shortcodes_TimeMajor',
                'timestart' => 'VTCore_Timeline_Shortcodes_TimeStart',
            ])
            ->initialize();
    }


    /**
     * Registering actions
     */
    protected function doActions()
    {

        // Loading additional assets for VC backend editor
        add_action('vc_backend_editor_enqueue_js_css', function() {
            VTCore_Wordpress_Utility::loadAsset('jquery-iconpicker');
            VTCore_Wordpress_Utility::loadAsset('font-awesome');
            VTCore_Wordpress_Utility::loadAsset('timeline-admin');
            VTCore_Wordpress_init::getFactory('assets')
                ->get('queues')
                ->remove('timeline-admin.js.timeline-admin-js');
        }, 10);


        // Loading additional assets for VC frontend editor
        add_action('vc_frontend_editor_enqueue_js_css', function() {
            VTCore_Wordpress_Utility::loadAsset('jquery-custom-scrollbar');
            VTCore_Wordpress_Utility::loadAsset('jquery-iconpicker');
            VTCore_Wordpress_Utility::loadAsset('timeline-front');
            VTCore_Wordpress_Utility::loadAsset('timeline-admin');
        }, 10);


        // Loading additional assets for VC frontend editor
        add_action('vc_load_iframe_jscss', function() {
            VTCore_Wordpress_Utility::loadAsset('jquery-custom-scrollbar');
            VTCore_Wordpress_Utility::loadAsset('jquery-iconpicker');
            VTCore_Wordpress_Utility::loadAsset('timeline-front');
            VTCore_Wordpress_Utility::loadAsset('timeline-admin');
        }, 10);


        // VTCore updater
        add_action('vtcore_updater', function($updater) {
            $updater
                ->execute(array(
                    'version' => VTCORE_TIMELINE_VERSION,
                    'object' => 'VTCore_Timeline_Updater',
                    'plugin' => 'victheme_timeline',
                ));
        }, 10);
    }


    /**
     * Registering filters
     */
    protected function doFilters()
    {
        // Adding VC default templates
        add_filter('vc_load_default_templates', function($templates) {
            array_unshift($templates, array(
                'name' => __('* TimeLine Elements', 'victheme_timeline'),
                'custom_class' => 'vtcore-timeline',
                'content' => '
                    [vc_row][vc_column][timeline][timestart]Ready Go[/timestart][timeevents datetime="10.10" day="Monday" month="August" year="2015" date="15" text="timeline" icon="angle-up" direction="right"][vc_column_text]I am text block. Click edit button to change this text. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.
                    [/vc_column_text][/timeevents][timemajor]Major Events[/timemajor][timeevents day="Saturday" month="November" year="2015" date="13" text="Timeline " icon="angle-down" direction="left"][vc_column_text]I am text block. Click edit button to change this text. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.
                    [/vc_column_text][/timeevents][timeevents day="Saturday" month="November" year="2015" date="13" text="Timeline " icon="angle-down" direction="right"][vc_column_text]I am text block. Click edit button to change this text. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.
                    [/vc_column_text][/timeevents][timeend direction="center"]Stop Here[/timeend][/timeline][/vc_column][/vc_row]'

            ));

            return $templates;
        }, 10);

        // Force convert shortcode into dotted notation, some theme like flatsome
        // invoke function that messed up with underscore (flatsome_contentfix)
        add_filter('the_content', function ($content) {
            $shortcodes = ['timeline', 'timelinesimple'];
            foreach ($shortcodes as $shortcode) {
                if (has_shortcode($content, $shortcode)) {
                    $pattern = get_shortcode_regex(array($shortcode));
                    $atts = [];
                    preg_match_all('/' . $pattern . '/s', $content, $atts);

                    if (!empty($atts[0])) {
                        $content = str_replace($atts[0], str_replace('___', '.', $atts[0]), $content);
                    }
                }

            }
            return $content;
        }, 999);
    }


    /**
     * Registering shortcodes
     */
    protected function doVisualComposer()
    {
        VTCore_Wordpress_Init::getFactory('visualcomposer')
            ->mapShortcode([
                'VTCore_Timeline_Composer_TimeLine',
                'VTCore_Timeline_Composer_TimeLineSimple',
                'VTCore_Timeline_Composer_TimeLineQuery',
                'VTCore_Timeline_Composer_TimeMajor',
                'VTCore_Timeline_Composer_TimeEvents',
                'VTCore_Timeline_Composer_TimeEnd',
                'VTCore_Timeline_Composer_TimeStart',
            ]);
    }
}

(new VTCore_Timeline);